/*

Volcano (abans "Rise of the Bal1")
Programat i dissenyat per
	Sergio Valor @JailDesigner
Editor, música i suport técnic per
	Raimon Zamora @JailDoc
Gràfics per
	Diego Valor @JailBrother
Començat un 19 de febrer de 2016
Repres un 14 de febrer de 2021

*/

#include "ifdefs.h"
#include <time.h>
#include <stdio.h>
#include <string>
#include "const.h"
#include "director.h"

int main(int argc, char *args[])
{
	// Inicia el generador de numeros aleatorios
	srand(time(nullptr));

	// Crea el objeto director
	Director *director = new Director(args[0]);

	/* 	// Comprueba si existen todos los ficheros de recursos
		if (!director->initResourceList())
			return -1;

		// Arranca SDL y crea la ventana
		if (!director->initSDL())
			return -1;

		director->initGame();
		while (director->isRunning())
		{
			director->runGame();
		}
		director->quitGame(); */

	// Bucle principal
	director->run();

	// Libera todos los recursos y cierra SDL
	delete director;
	printf("Shutting down the game...\n");

	return 0;
}
